from OpenGL.GL import *
from OpenGL.GLU import *
import random
from math import * # trigonometry

import pygame, sys,os, time, random
from pygame.locals import *

from GLTools import *

class Application:
    instance = None
    current_time = 0
    
    def __init__(self):
        if Application.instance != None:
            raise Exception("Application instance duplicated")
        
        self.mouse = {'pos':[0,0],'buttons':[0,0,0],'delta':[0,0]}
        self.keys = []
        Application.instance = self

    def init(self):
        glDisable( GL_CULL_FACE )        

    def createWindow(self, caption, size, fullscreen = False, resizable = False, frame = True):

        pygame.init()
        self.flags = pygame.OPENGL|pygame.DOUBLEBUF
        if resizable:
            self.flags |= pygame.RESIZABLE
        if not frame:
            self.flags |= pygame.NOFRAME
        if fullscreen:
            self.flags |= pygame.FULLSCREEN

        self.size = size
        pygame.display.set_caption(caption)
        pygame.display.set_mode(self.size[:2],self.flags)

    def render(self):
        glClearColor(0.0, 0.0, 0.0, 1.0)
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

        #render scene...

    def update(self, time_in_ms):
        pass

    def quit(self):
        pygame.display.quit()
        print "********** Application End ***********"
        os._exit(1)     

    def onKeyPressed(self,key):
        if key == pygame.K_ESCAPE: 
            self.quit()
        return False

    def onKeyReleased(self,key):
        pass
    def onMouseButtonPressed(self,button):
        pass
    def onMouseButtonReleased(self,button):
        pass

    def process_events(self, events):
        for event in events: 
            if event.type == QUIT:
                self.quit()
            if event.type == pygame.KEYDOWN:
                self.onKeyPressed(event.key)
            elif event.type == pygame.KEYUP:
                self.onKeyReleased(event.key)
            elif event.type == pygame.MOUSEBUTTONDOWN:
                self.onMouseButtonPressed(event.button)
            elif event.type == pygame.MOUSEBUTTONUP:
                self.onMouseButtonReleased(event.button)

    def mainLoop(self):
        self.last_time = pygame.time.get_ticks()
        self.elapsed_time = 0

        while(True):
            self.render()
            pygame.display.flip() #swap
            
            self.current_time = pygame.time.get_ticks()
            self.elapsed_time = max(self.current_time - self.last_time,0.001)
            self.last_time = self.current_time
            pygame.event.pump()
            self.keys=pygame.key.get_pressed()

            prev = self.mouse['pos']
            self.mouse['pos'] = pygame.mouse.get_pos()
            self.mouse['buttons'] = pygame.mouse.get_pressed()
            self.mouse['delta'] = [ self.mouse['pos'][0] - prev[0], self.mouse['pos'][1] - prev[1]]
            
            self.update(self.elapsed_time)

            self.process_events( pygame.event.get() ) #sync input

    def start(self):
        try:
            self.init()
            self.mainLoop()                
        except:
            print "Exception: "
            traceback.print_exc()
            print "*"*60
            self.quit()

    def takeScreenshot(self):
        enableRenderHUD(0,self.size[0], self.size[1], 0)
        self.canvas.render(self.size)
        disableRenderHUD()
        jpg_file_write("screenshots/my_image",self.current_time,self.size)              

    def renderFPSbar(self,elapsed_time_in_ms):
        if (elapsed_time_in_ms == 0): elapsed_time_in_ms = 1
        total = 0.01666 / float(elapsed_time_in_ms * 0.001)

        enableRenderHUD(0,1,0,1)

        glDisable( GL_BLEND )
        glDisable( GL_TEXTURE_2D )
        glLineWidth(5)
        glBegin(GL_LINES)
        glColor3f(0.5,0.5,0.7)
        glVertex3f(-1,0,0)
        glVertex3f(-1 + 2 * total,0,0)
        glEnd()

        disableRenderHUD()        
            
        
