#!/usr/local/bin/python

# Pyncel App coded by Javi Agenjo javi.agenjo@gmail.com 2009

from OpenGL.GL import *
from OpenGL.GLU import *

from GLTools import *
from canvas import *
from utils import *
import random

from Application import *
from math import * # trigonometry

WINDOW_SIZE = [512,512,False]
WINDOW_SIZE = [800,600,False]
#WINDOW_SIZE = [1440,900,True]
#WINDOW_SIZE = [1650,1050,True]

class MyApp(Application):

    flags = {'cicling_colors':True,'drawing':False,'dragging':False,'undo':True }
    brushes = []
    scale = 1.0

    def init(self):
        glDisable( GL_CULL_FACE )        
        glDisable( GL_DEPTH_TEST )
        self.logo_tex = Texture()
        self.logo_tex.load("data/tmt-logo.png")

        self.canvas = [] 
        self.canvas.append( Canvas([1024,1024]) ) #self.size)
        #self.canvas.append( Canvas([512,512]) ) #self.size)
        #self.canvas[1].increaseOffset(vec([self.canvas[1].size[0],0]))
        self.offset = ( vec(WINDOW_SIZE[:2]) - vec( self.canvas[0].size) ) * 0.5
        
        self.brushes.append( Brush(["data/brush-circle2.tga"] ) )
        self.brushes.append( Brush(["data/brush.png"] ) )
        self.brushes.append( Brush(["data/brush-circle.png"] ) )
        self.brushes.append( Brush(["data/cloud1.png","data/cloud2.png","data/cloud3.png"] ) )
        #self.brushes.append( Brush(["data/star-1.png", "data/star-2.png", "data/star-3.png", "data/star-4.png", "data/star-5.png" ]) )
        self.brushes.append( Brush(["data/stencil-mrt.png", "data/stencil-obama.png", "data/stencil-house.png"] ) )
        print "Brushes loaded: " + str( len(self.brushes) )
        self.brush = self.brushes[0]

    def render(self):
        glClearColor(0.0, 0.0, 0.0, 1.0)
        glClear(GL_COLOR_BUFFER_BIT)

        setCamera2D(0,self.size[0], self.size[1], 0)

        glPushMatrix()
        glTranslatef(self.offset[0], self.offset[1], 0.0)
        glScale(self.scale,self.scale,1.0)

        for c in self.canvas:
            c.render(self.size)
        self.brush.renderInPos()
        glPopMatrix()

        self.renderHUD()

    def renderHUD(self):
        setCamera2D(0,self.size[0], self.size[1], 0)
        
        glColor4f(0.03,0.06,0.09,0.5)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        glDisable( GL_TEXTURE_2D )
        drawQuad([0,self.size[1]-25],[self.size[0],25])
        glBlendFunc(GL_ONE,GL_ONE)
        glColor4f(0.3,0.3,0.3,0.5)
        printGL("Preassure: " + "%3.3f" % self.brush.preassure,[10,self.size[1]-10])
        printGL("Flow: " + "%3.3f" % (1000.0 / self.brush.time_interval_in_ms),[180,self.size[1]-10])
        printGL("Texture: " + self.brush.textures[0].filename,[340,self.size[1]-10])
        printGL("Angle: " + str(self.brush.angle),[640,self.size[1]-10])

        self.renderFPSbar(self.elapsed_time)

    def update(self, time_in_ms):

        self.brush.setPosition( self.project(self.mouse['pos'][:2]) )

        for c in self.canvas:
            c.update()

        if MyApp.flags['drawing']:
            for c in self.canvas:
                self.brush.paint( c )

        if MyApp.flags['dragging']:
            for c in self.canvas:
                c.increaseOffset( vec(self.mouse['delta']) / self.scale)

        #self.canvas[0],self.canvas[1]=self.canvas[1],self.canvas[0]
            
        #print "-"*30

        if MyApp.flags['cicling_colors']:
            self.brush.color = [ sin(self.current_time * 0.001)*0.5+0.5, 0.3, cos(self.current_time * 0.001)*0.5]

        if self.keys[pygame.K_KP_MULTIPLY]:
            self.brush.time_interval_in_ms *= 1.01
        if self.keys[pygame.K_KP_DIVIDE]:
            self.brush.time_interval_in_ms *= 0.99
        if self.keys[pygame.K_KP_PLUS]:
            self.brush.preassure *= 1.01 if not self.keys[pygame.K_LCTRL] else 1.1
        if self.keys[pygame.K_KP_MINUS]:
            self.brush.preassure *= 0.99 if not self.keys[pygame.K_LCTRL] else 0.9

        #print "Stack: " + str( glGetIntegerv(GL_MODELVIEW_STACK_DEPTH) )

    def setScale(self,value):
        self.scale = value
        self.offset = ( vec(WINDOW_SIZE[:2]) - vec(self.canvas[0].size) * self.scale ) * 0.5

    def project(self,pos):
        # self.brush.position = vec(pos) - ( vec(WINDOW_SIZE[:2]) - vec(self.canvas.size) ) * 0.5
        return ( vec(pos) - self.offset) / self.scale

    def onMouseButtonPressed(self,button):
        #print "button: "+str(button)
        if button == 1: #left
            if MyApp.flags['undo']:
                for c in self.canvas:
                    c.saveUndo()
            MyApp.flags['drawing'] = True

        if button == 3: #right
            MyApp.flags['dragging'] = True

        if self.keys[pygame.K_LSHIFT]:
            if button == 4: #wheelup
                self.brush.brush_rotation_mode = Brush.NORMAL
                self.brush.angle = (self.brush.angle + 5) % 360
            elif button == 5: #wheeldown
                self.brush.brush_rotation_mode = Brush.NORMAL
                self.brush.angle = (self.brush.angle - 5) % 360
        elif self.keys[pygame.K_LCTRL]:
            if button == 2: #middle
                self.setScale(1.0)
            elif button == 4: #wheelup
                self.setScale( self.scale * 1.1 )
            elif button == 5: #wheeldown
                self.setScale( self.scale * 0.9 )        
        else:
            if button == 4: #wheelup
                self.brush.size = [i*1.1 for i in self.brush.size]
            elif button == 5: #wheeldown
                self.brush.size = [i*0.9 for i in self.brush.size]

    def onMouseButtonReleased(self,button):
        if button == 1: #left
            MyApp.flags['drawing'] = False
            self.brush.last_draw_time = 0
        if button == 3: #right
            MyApp.flags['dragging'] = False

    def onKeyPressed(self,key):
        Application.onKeyPressed(self,key)
        if key == pygame.K_s and self.keys[pygame.K_LCTRL]:
            for c in self.canvas:
                filename = ChooseFileDialog()
                #filename = "screenshots/my_image"+str(self.current_time)+"_"+str(id(c))+".png"
                if c.rt.save(filename):
                    print "Saved: "+filename
            
        if key == pygame.K_c:
            for c in self.canvas:
                c.clear()
        if key == pygame.K_t:
            self.brush.brush_texture_mode = Brush.NORMAL
        if key == pygame.K_r:
            self.brush.brush_rotation_mode = Brush.RANDOM
        if key == pygame.K_b:
            self.brush.additive_blend = not self.brush.additive_blend
        if key == pygame.K_LALT:
            pass
        if key == pygame.K_KP_ENTER:
            self.brush.incTexture()
        if key == pygame.K_g:
            Canvas.render_grid = not Canvas.render_grid 
        if key == pygame.K_KP0:
            self.brush.color = [0.0,0.0,0.0,1.0]
            self.brush.additive_blend = False
            MyApp.flags['cicling_colors'] = False
        if key == pygame.K_KP_PERIOD:
            MyApp.flags['cicling_colors'] = not MyApp.flags['cicling_colors']
            if not MyApp.flags['cicling_colors']:
                self.brush.color = [1.0]*4
        if key == pygame.K_z and self.keys[pygame.K_LCTRL]:
            for c in self.canvas:
                c.recoverUndo()
        if key == pygame.K_1: self.brush = self.brushes[0]
        if key == pygame.K_2 and len(self.brushes) > 1: self.brush = self.brushes[1]
        if key == pygame.K_3 and len(self.brushes) > 2: self.brush = self.brushes[2]
        if key == pygame.K_4 and len(self.brushes) > 3: self.brush = self.brushes[3]
        if key == pygame.K_5 and len(self.brushes) > 4: self.brush = self.brushes[4]


app = MyApp()
app.createWindow("My App", WINDOW_SIZE, WINDOW_SIZE[2] )
app.start()
